/*
 * Copyright (C) 2004 Domenico Delle Side
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "getphoto.h"

#include <unistd.h>
#include <pwd.h>
#include <sys/types.h>


int change_user(const char *filename)
     /* 
      * getphoto must be setuid root to work properly; so root is the
      * owner of every file copied, despite the user that runs the
      * program is not the super user.  
      *
      * As a workaround, we change the owner to that returned by
      * getuid().
      *
      */
{
  struct passwd *running_user;

  /* 
   * From getpwuid()'s man page:
   *
   * RETURN VALUE
   *
   * The getpwnam() and getpwuid() functions return a pointer to the
   * passwd structure, or NULL if the matching entry is not found or
   * an error occurs.  If an error occurs, errno is set appropriately.
   * If one wants to check errno after the call, it should be set to
   * zero before the call.
   *
   * So, we do it.
   *
   */

  errno = 0;
  running_user = getpwuid(getuid());

  if ( NULL == running_user )
    {
      print_error(GT_("Errors while changing ownership: %m\n"));
      return -1;
    }

  if ( 0 > chown(filename, running_user->pw_uid, 
		 running_user->pw_gid) )
    {
      print_error(GT_("Errors while changing ownership: %m\n"));
      return -1;
    }

  return 1;
}
