/*
 * Copyright (C) 2004 Domenico Delle Side
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "getphoto.h"


static FILE *open_conf(char *path)
{
  FILE *conf_file = NULL;

  if ( NULL == (conf_file = fopen(path, "r")) )
    exit_errno();

  return conf_file;
}


static char *check_line(FILE *fp)
{
  char buf[MAXLEN];
                                                                                                               
  do {
    if ( NULL == fgets(buf, MAXLEN, fp) )
      return NULL;
  } while ( '#' == buf[0] || '\n' == buf[0] );

  return strdup(buf);
}


static char *get_value(char *s)
{
  char *equal, *line, *tmp, *res;

  line = strdup(s);
  
  if ( NULL == (equal = strchr(line, '=')) )
    print_exit(GT_("Errors parsing configuration file\n"));
  else
    equal++;

  while ( ' ' == *equal )
    equal++;

  tmp = equal;

  while ( !isspace(*tmp) )
    if ( '#' == *tmp )
      break;
    else
      tmp++;

  res = mystrndup(equal, (tmp - equal));
  free(line);

  return res;
}


static int get_numeric_value(char *s)
{
  char *tmp;
  int value;

  tmp = get_value(s);

  if ( !isdigit(*tmp) )
    print_exit(GT_("Errors parsing configuration file\n"));

  value = atoi(tmp);
  free(tmp);
  
  return value;
}


static struct config *
set_options(char *line, struct config *conf)
{

  if ( NULL == conf )
    conf = Calloc(1, sizeof(struct config));

  while ( isspace(*line) )
    line++;

  if ( 0 == strncmp("device", line, strlen("device")) )
    conf->device = get_value(line);
  else if ( 0 == strncmp("fs_type", line, strlen("fs_type")) )
    conf->fs_type = get_value(line);
  else if ( 0 == strncmp("mount_point", line, strlen("mount_point")) )
    conf->mount_point = get_value(line);
  else if ( 0 == strncmp("photo_path", line, strlen("photo_path")) )
    conf->photo_path = get_value(line);
  else if ( 0 == strncmp("dest_dir", line, strlen("dest_dir")) )
    conf->dest_dir = get_value(line);
  else if ( 0 == strncmp("preserve", line, strlen("preserve")) )
    conf->preserve = get_numeric_value(line);
  else if ( 0 == strncmp("usedate", line, strlen("usedate")) )
    conf->usedate = get_numeric_value(line);
#ifdef HAVE_JPEGLIB_H
  else if ( 0 == strncmp("resize", line, strlen("resize")) )
    conf->resize = get_numeric_value(line);
  else if ( 0 == strncmp("quality", line, strlen("quality")) )
    conf->img.img_quality = get_numeric_value(line);
  else if ( 0 == strncmp("width", line, strlen("width")) )
    conf->img.img_w = get_numeric_value(line);
  else if ( 0 == strncmp("height", line, strlen("height")) )
    conf->img.img_h = get_numeric_value(line);
#endif
  else
    return NULL;
  
  return conf;
}


struct config *parse_config(char *filename)
{
  FILE *config;
  char *line;
  struct config *conf = NULL;

  config = open_conf(filename);

  while ( NULL != (line = check_line(config)) )
    if ( NULL == (conf = set_options(line, conf)) )
      print_exit(GT_("Errors parsing configuration file\n"));

  fclose(config);

  return conf;
}


void free_options(struct config *conf)
{
  free(conf->device);
  free(conf->mount_point);
  free(conf->photo_path);
  free(conf->dest_dir);
  free(conf);
}
